<div class="ipynb-field">
  <div class="form-group">
    {% if this.args.description %}
      <p>{{ this.args.description }}</p>
    {% endif %}
    <label for="{{ this.args.name }}">
      {{ this.args.label }}
    </label>
    <select class="select2" name="{{ this.args.name }}">
    {% for choice in this.args.choices %}
      <option{% if choice == this.args.default %} selected{% endif %}>{{ choice }}</option>
    {% endfor %}
    </select>
  </div>
</div>