{% extends "default/form.j2" %}

{% block body %}
<div id="_content" class="container loading">
  <div class="text-center"><div class="lds-ellipsis"><div></div><div></div><div></div><div></div></div></div>
  <script>
    require(['svelte'], function(svelte) {
      svelte.render(
        document.getElementById("_content"),
        '{{ url_for('static', filename='js/form.js') }}',
        JSON.parse(atob('{{ ({ 'fields': _fields, 'extras': _config.EXTRAS, 'debug': _config.DEBUG })|jsonify|btoa }}'))
      )
    })
  </script>
</div>
{% endblock %}

{% block script %}
{{ super() }}
<script>
require(['jquery', 'bootstrap'], function ($) {
  $(document).ready(function () {
    $('.collapse').collapse({ toggle: false })
    $('[data-toggle="tooltip"]').tooltip({ boundary: 'window' })
  })
})
</script>
{% endblock %}
